@@ -10,6 +10,7 @@ from cyvcf2 import VCF, Variant
 from pysam import VariantFile
 
 from haptools import data
+from .logging import getLogger
 
 
 @dataclass
@@ -176,7 +177,10 @@ class GenotypesAncestry(data.GenotypesRefAlt):
         super().__init__(fname, log)
         self.ancestry = None
         self.valid_labels = None
+        # goes from population code to encoding number
         self.ancestry_labels = {}
+        # goes from encoding number to population code
+        self.popnum_ancestry = {}
 
     def _iterate(self, vcf: VCF, region: str = None, variants: set[str] = None):
         """
@@ -210,6 +214,7 @@ class GenotypesAncestry(data.GenotypesRefAlt):
                 for pop in pops:
                     if pop not in self.ancestry_labels:
                         self.ancestry_labels[pop] = pop_count
+                        self.popnum_ancestry[pop_count] = pop
                         pop_count += 1
                 ancestry[i] = tuple(map(self.ancestry_labels.get, pops))
             # finally, output everything
@@ -264,7 +269,7 @@ class GenotypesAncestry(data.GenotypesRefAlt):
             )
             self.valid_labels = np.empty(
                 (max_variants, len(self.samples), 2),
-                dtype=np.string_,
+                dtype=object,
             )
             num_seen = 0
             for rec in records:
@@ -438,27 +443,32 @@ class GenotypesAncestry(data.GenotypesRefAlt):
                 ("Description", "Genotype"),
             ],
         )
-        vcf.header.add_meta(
-            "FORMAT",
-            items=[
-                ("ID", "POP"),
-                ("Number", 2),
-                ("Type", "String"),
-                ("Description", "Origin Population of each respective allele in GT"),
-            ],
-        )
-        vcf.header.add_meta(
-            "FORMAT",
-            items=[
-                ("ID", "SAMPLE"),
-                ("Number", 2),
-                ("Type", "String"),
-                (
-                    "Description",
-                    "Origin sample and haplotype of each respective allele in GT",
-                ),
-            ],
-        )
+        if not self.ancestry is None:
+            vcf.header.add_meta(
+                "FORMAT",
+                items=[
+                    ("ID", "POP"),
+                    ("Number", 2),
+                    ("Type", "String"),
+                    (
+                        "Description",
+                        "Origin Population of each respective allele in GT",
+                    ),
+                ],
+            )
+        if not self.valid_labels is None:
+            vcf.header.add_meta(
+                "FORMAT",
+                items=[
+                    ("ID", "SAMPLE"),
+                    ("Number", 2),
+                    ("Type", "String"),
+                    (
+                        "Description",
+                        "Origin sample and haplotype of each respective allele in GT",
+                    ),
+                ],
+            )
         try:
             vcf.header.add_samples(self.samples)
         except AttributeError:
@@ -486,18 +496,22 @@ class GenotypesAncestry(data.GenotypesRefAlt):
             for samp_idx, sample in enumerate(self.samples):
                 # TODO: make this work when there are missing values
                 record.samples[sample]["GT"] = tuple(self.data[samp_idx, var_idx, :2])
-                record.samples[sample]["POP"] = tuple(
-                    self.ancestry[samp_idx, var_idx, :]
-                )
-                record.samples[sample]["SAMPLE"] = tuple(
-                    self.valid_labels[samp_idx, var_idx, :]
-                )
+                if not self.ancestry is None:
+                    record.samples[sample]["POP"] = tuple(
+                        map(
+                            self.popnum_ancestry.get,
+                            self.ancestry[samp_idx, var_idx, :],
+                        )
+                    )
+                if not self.valid_labels is None:
+                    record.samples[sample]["SAMPLE"] = tuple(
+                        self.valid_labels[samp_idx, var_idx, :]
+                    )
                 # TODO: add proper phase info
                 record.samples[sample].phased = True
             # write the record to a file
             vcf.write(record)
         vcf.close()
-        raise ValueError("Not implemented")
 
 
 def transform_haps(
@@ -549,11 +563,7 @@ def transform_haps(
         A logging module to which to write messages about progress and any errors
     """
     if log is None:
-        log = logging.getLogger("haptools transform")
-        logging.basicConfig(
-            format="[%(levelname)8s] %(message)s (%(filename)s:%(lineno)s)",
-            level="ERROR",
-        )
+        log = getLogger(name="transform", level="ERROR")
 
     haps_class = HaplotypesAncestry if ancestry else data.Haplotypes
     log.info("Loading haplotypes")
